@@ -114,6 +114,11 @@ fn main() {
             src_dir: "vendor/tree-sitter-haskell-src",
             extra_files: vec!["scanner.cc"],
         },
+        TreeSitterParser {
+            name: "tree-sitter-janet-simple",
+            src_dir: "vendor/tree-sitter-janet-simple-src",
+            extra_files: vec!["scanner.c"],
+        },
         TreeSitterParser {
             name: "tree-sitter-java",
             src_dir: "vendor/tree-sitter-java-src",
@@ -129,6 +134,16 @@ fn main() {
             src_dir: "vendor/tree-sitter-json-src",
             extra_files: vec![],
         },
+        TreeSitterParser {
+            name: "tree-sitter-lua",
+            src_dir: "vendor/tree-sitter-lua-src",
+            extra_files: vec!["scanner.cc"],
+        },
+        TreeSitterParser {
+            name: "tree-sitter-nix",
+            src_dir: "vendor/tree-sitter-nix-src",
+            extra_files: vec!["scanner.c"],
+        },
         TreeSitterParser {
             name: "tree-sitter-ocaml",
             src_dir: "vendor/tree-sitter-ocaml-src/ocaml/src",
