@@ -55,9 +55,12 @@ extern "C" {
     fn tree_sitter_elixir() -> ts::Language;
     fn tree_sitter_go() -> ts::Language;
     fn tree_sitter_haskell() -> ts::Language;
+    fn tree_sitter_janet_simple() -> ts::Language;
     fn tree_sitter_java() -> ts::Language;
     fn tree_sitter_javascript() -> ts::Language;
     fn tree_sitter_json() -> ts::Language;
+    fn tree_sitter_lua() -> ts::Language;
+    fn tree_sitter_nix() -> ts::Language;
     fn tree_sitter_ocaml() -> ts::Language;
     fn tree_sitter_ocaml_interface() -> ts::Language;
     fn tree_sitter_php() -> ts::Language;
@@ -87,10 +90,10 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Bash",
                 language,
-                atom_nodes: (vec!["string", "raw_string", "heredoc_body"])
+                atom_nodes: vec!["string", "raw_string", "heredoc_body"]
                     .into_iter()
                     .collect(),
-                delimiter_tokens: (vec![("(", ")"), ("{", "}"), ("[", "]")]),
+                delimiter_tokens: vec![("(", ")"), ("{", "}"), ("[", "]")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/bash.scm"),
@@ -103,10 +106,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "C",
                 language,
-                atom_nodes: (vec!["string_literal", "char_literal"])
-                    .into_iter()
-                    .collect(),
-                delimiter_tokens: (vec![("(", ")"), ("{", "}"), ("[", "]")]),
+                atom_nodes: vec!["string_literal", "char_literal"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("{", "}"), ("[", "]")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/c.scm"),
@@ -121,10 +122,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 language,
                 // The C++ grammar extends the C grammar, so the node
                 // names are generally the same.
-                atom_nodes: (vec!["string_literal", "char_literal"])
-                    .into_iter()
-                    .collect(),
-                delimiter_tokens: (vec![("(", ")"), ("{", "}"), ("[", "]"), ("<", ">")]),
+                atom_nodes: vec!["string_literal", "char_literal"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("{", "}"), ("[", "]"), ("<", ">")],
                 highlight_query: ts::Query::new(
                     language,
                     concat!(
@@ -140,8 +139,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Clojure",
                 language,
-                atom_nodes: (vec![]).into_iter().collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")"), ("[", "]")])
+                atom_nodes: vec![].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]")]
                     .into_iter()
                     .collect(),
                 highlight_query: ts::Query::new(
@@ -156,8 +155,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Common Lisp",
                 language,
-                atom_nodes: (vec!["str_lit", "char_lit"]).into_iter().collect(),
-                delimiter_tokens: (vec![("(", ")")]),
+                atom_nodes: vec!["str_lit", "char_lit"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")")],
                 highlight_query: ts::Query::new(language, "").unwrap(),
             }
         }
@@ -166,14 +165,14 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "C#",
                 language,
-                atom_nodes: (vec![
+                atom_nodes: vec![
                     "string_literal",
                     "verbatim_string_literal",
                     "character_literal",
-                ])
+                ]
                 .into_iter()
                 .collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")")]),
+                delimiter_tokens: vec![("{", "}"), ("(", ")")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/c-sharp.scm"),
@@ -186,8 +185,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "CSS",
                 language,
-                atom_nodes: (vec!["integer_value", "float_value"]).into_iter().collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")")]),
+                atom_nodes: vec!["integer_value", "float_value"].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/css.scm"),
@@ -200,8 +199,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Dart",
                 language,
-                atom_nodes: (vec!["string_literal", "script_tag"]).into_iter().collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")"), ("[", "]"), ("<", ">")]),
+                atom_nodes: vec!["string_literal", "script_tag"].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]"), ("<", ">")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/dart.scm"),
@@ -214,8 +213,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Emacs Lisp",
                 language,
-                atom_nodes: (vec![]).into_iter().collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")"), ("[", "]")])
+                atom_nodes: vec![].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]")]
                     .into_iter()
                     .collect(),
                 highlight_query: ts::Query::new(
@@ -230,8 +229,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Elixir",
                 language,
-                atom_nodes: (vec!["string", "heredoc"]).into_iter().collect(),
-                delimiter_tokens: (vec![("(", ")"), ("{", "}"), ("do", "end")])
+                atom_nodes: vec!["string", "heredoc"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("{", "}"), ("do", "end")]
                     .into_iter()
                     .collect(),
                 highlight_query: ts::Query::new(
@@ -246,10 +245,10 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Go",
                 language,
-                atom_nodes: (vec!["interpreted_string_literal", "raw_string_literal"])
+                atom_nodes: vec!["interpreted_string_literal", "raw_string_literal"]
                     .into_iter()
                     .collect(),
-                delimiter_tokens: (vec![("{", "}"), ("[", "]"), ("(", ")")])
+                delimiter_tokens: vec![("{", "}"), ("[", "]"), ("(", ")")]
                     .into_iter()
                     .collect(),
                 highlight_query: ts::Query::new(
@@ -264,8 +263,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Haskell",
                 language,
-                atom_nodes: (vec![]).into_iter().collect(),
-                delimiter_tokens: (vec![("[", "]"), ("(", ")")]),
+                atom_nodes: vec![].into_iter().collect(),
+                delimiter_tokens: vec![("[", "]"), ("(", ")")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/haskell.scm"),
@@ -273,13 +272,36 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 .unwrap(),
             }
         }
+        JanetSimple => {
+            let language = unsafe { tree_sitter_janet_simple() };
+            TreeSitterConfig {
+                name: "Janet",
+                language,
+                atom_nodes: vec![].into_iter().collect(),
+                delimiter_tokens: vec![
+                    ("@{", "}"),
+                    ("@(", ")"),
+                    ("@[", "]"),
+                    ("{", "}"),
+                    ("(", ")"),
+                    ("[", "]"),
+                ]
+                .into_iter()
+                .collect(),
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/janet_simple.scm"),
+                )
+                .unwrap(),
+            }
+        }
         Java => {
             let language = unsafe { tree_sitter_java() };
             TreeSitterConfig {
                 name: "Java",
                 language,
-                atom_nodes: (vec![]).into_iter().collect(),
-                delimiter_tokens: (vec![("(", ")"), ("{", "}")]),
+                atom_nodes: vec![].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("{", "}"), ("[", "]")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/java.scm"),
@@ -292,10 +314,10 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "JavaScript",
                 language,
-                atom_nodes: (vec!["string", "template_string", "regex"])
+                atom_nodes: vec!["string", "template_string", "regex"]
                     .into_iter()
                     .collect(),
-                delimiter_tokens: (vec![
+                delimiter_tokens: vec![
                     ("[", "]"),
                     ("(", ")"),
                     ("{", "}"),
@@ -303,7 +325,7 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                     // like 1 < 2, but we should never see both a < and a
                     // > at the same level in JSX.
                     ("<", ">"),
-                ]),
+                ],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/javascript.scm"),
@@ -316,8 +338,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "JSON",
                 language,
-                atom_nodes: (vec!["string"]).into_iter().collect(),
-                delimiter_tokens: (vec![("{", "}"), ("[", "]")]),
+                atom_nodes: vec!["string"].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("[", "]")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/json.scm"),
@@ -325,13 +347,45 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 .unwrap(),
             }
         }
+        Lua => {
+            let language = unsafe { tree_sitter_lua() };
+            TreeSitterConfig {
+                name: "Lua",
+                language,
+                atom_nodes: vec!["string"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("{", "}"), ("[", "]")]
+                    .into_iter()
+                    .collect(),
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/lua.scm"),
+                )
+                .unwrap(),
+            }
+        }
+        Nix => {
+            let language = unsafe { tree_sitter_nix() };
+            TreeSitterConfig {
+                name: "Nix",
+                language,
+                atom_nodes: vec!["string_expression", "indented_string_expression"]
+                    .into_iter()
+                    .collect(),
+                delimiter_tokens: vec![("{", "}"), ("[", "]")].into_iter().collect(),
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/nix.scm"),
+                )
+                .unwrap(),
+            }
+        }
         OCaml => {
             let language = unsafe { tree_sitter_ocaml() };
             TreeSitterConfig {
                 name: "OCaml",
                 language,
                 atom_nodes: OCAML_ATOM_NODES.iter().copied().collect(),
-                delimiter_tokens: (vec![("(", ")"), ("[", "]"), ("{", "}")]),
+                delimiter_tokens: vec![("(", ")"), ("[", "]"), ("{", "}")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/ocaml.scm"),
@@ -345,7 +399,7 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 name: "OCaml Interface",
                 language,
                 atom_nodes: OCAML_ATOM_NODES.iter().copied().collect(),
-                delimiter_tokens: (vec![("(", ")"), ("[", "]"), ("{", "}")]),
+                delimiter_tokens: vec![("(", ")"), ("[", "]"), ("{", "}")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/ocaml.scm"),
@@ -358,8 +412,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "PHP",
                 language,
-                atom_nodes: (vec!["string"]).into_iter().collect(),
-                delimiter_tokens: (vec![("(", ")"), ("[", "]"), ("{", "}")]),
+                atom_nodes: vec!["string", "encapsed_string"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("[", "]"), ("{", "}")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/php.scm"),
@@ -372,8 +426,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Python",
                 language,
-                atom_nodes: (vec!["string"]).into_iter().collect(),
-                delimiter_tokens: (vec![("(", ")"), ("[", "]"), ("{", "}")]),
+                atom_nodes: vec!["string"].into_iter().collect(),
+                delimiter_tokens: vec![("(", ")"), ("[", "]"), ("{", "}")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/python.scm"),
@@ -386,10 +440,10 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Ruby",
                 language,
-                atom_nodes: (vec!["string", "heredoc_body", "regex"])
+                atom_nodes: vec!["string", "heredoc_body", "regex"]
                     .into_iter()
                     .collect(),
-                delimiter_tokens: (vec![
+                delimiter_tokens: vec![
                     ("{", "}"),
                     ("(", ")"),
                     ("[", "]"),
@@ -397,7 +451,7 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                     ("def", "end"),
                     ("begin", "end"),
                     ("class", "end"),
-                ]),
+                ],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/ruby.scm"),
@@ -410,16 +464,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Rust",
                 language,
-                atom_nodes: (vec!["char_literal", "string_literal"])
-                    .into_iter()
-                    .collect(),
-                delimiter_tokens: (vec![
-                    ("{", "}"),
-                    ("(", ")"),
-                    ("[", "]"),
-                    ("|", "|"),
-                    ("<", ">"),
-                ]),
+                atom_nodes: vec!["char_literal", "string_literal"].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]"), ("|", "|"), ("<", ">")],
                 highlight_query: ts::Query::new(
                     language,
                     include_str!("../vendor/highlights/rust.scm"),
@@ -432,9 +478,13 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "Scala",
                 language,
-                atom_nodes: (vec!["string", "template_string"]).into_iter().collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")"), ("[", "]")]),
-                highlight_query: ts::Query::new(language, "").unwrap(),
+                atom_nodes: vec!["string", "template_string"].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]")],
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../vendor/highlights/scala.scm"),
+                )
+                .unwrap(),
             }
         }
         TypeScript => {
@@ -442,10 +492,10 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "TypeScript",
                 language,
-                atom_nodes: (vec!["string", "template_string", "regex"])
+                atom_nodes: vec!["string", "template_string", "regex"]
                     .into_iter()
                     .collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")"), ("[", "]"), ("<", ">")]),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]"), ("<", ">")],
                 highlight_query: ts::Query::new(
                     language,
                     concat!(
@@ -461,8 +511,8 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             TreeSitterConfig {
                 name: "TypeScript TSX",
                 language,
-                atom_nodes: (vec!["string", "template_string"]).into_iter().collect(),
-                delimiter_tokens: (vec![("{", "}"), ("(", ")"), ("[", "]"), ("<", ">")]),
+                atom_nodes: vec!["string", "template_string"].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("(", ")"), ("[", "]"), ("<", ">")],
                 highlight_query: ts::Query::new(
                     language,
                     concat!(
@@ -571,9 +621,18 @@ fn print_cursor(src: &str, cursor: &mut ts::TreeCursor, depth: usize) {
 
         if node.child_count() == 0 {
             let node_src = &src[node.start_byte()..node.end_byte()];
-            println!("{}{:#?} {:?}", "  ".repeat(depth), node, node_src);
+            println!(
+                "{}{} {:?}",
+                "  ".repeat(depth),
+                format!("{:?}", node).replace('\n', "\\n"),
+                node_src
+            );
         } else {
-            println!("{}{:#?}", "  ".repeat(depth), node);
+            println!(
+                "{}{}",
+                "  ".repeat(depth),
+                format!("{:?}", node).replace('\n', "\\n"),
+            );
         }
 
         if cursor.goto_first_child() {
@@ -682,7 +741,7 @@ fn all_syntaxes_from_cursor<'a>(
     let mut result: Vec<&Syntax> = vec![];
 
     loop {
-        result.push(syntax_from_cursor(
+        result.extend(syntax_from_cursor(
             arena, src, nl_pos, cursor, config, highlights,
         ));
 
@@ -703,7 +762,7 @@ fn syntax_from_cursor<'a>(
     cursor: &mut ts::TreeCursor,
     config: &TreeSitterConfig,
     highlights: &HighlightedNodeIds,
-) -> &'a Syntax<'a> {
+) -> Vec<&'a Syntax<'a>> {
     let node = cursor.node();
 
     if node.is_error() {
@@ -722,7 +781,9 @@ fn syntax_from_cursor<'a>(
         // of whether they have children.
         atom_from_cursor(arena, src, nl_pos, cursor, highlights)
     } else if node.child_count() > 0 {
-        list_from_cursor(arena, src, nl_pos, cursor, config, highlights)
+        vec![list_from_cursor(
+            arena, src, nl_pos, cursor, config, highlights,
+        )]
     } else {
         atom_from_cursor(arena, src, nl_pos, cursor, highlights)
     }
@@ -779,21 +840,21 @@ fn list_from_cursor<'a>(
     loop {
         let node = cursor.node();
         if node_i < i {
-            before_delim.push(syntax_from_cursor(
+            before_delim.extend(syntax_from_cursor(
                 arena, src, nl_pos, cursor, config, highlights,
             ));
         } else if node_i == i {
             inner_open_content = &src[node.start_byte()..node.end_byte()];
             inner_open_position = nl_pos.from_offsets(node.start_byte(), node.end_byte());
         } else if node_i < j {
-            between_delim.push(syntax_from_cursor(
+            between_delim.extend(syntax_from_cursor(
                 arena, src, nl_pos, cursor, config, highlights,
             ));
         } else if node_i == j {
             inner_close_content = &src[node.start_byte()..node.end_byte()];
             inner_close_position = nl_pos.from_offsets(node.start_byte(), node.end_byte());
         } else if node_i > j {
-            after_delim.push(syntax_from_cursor(
+            after_delim.extend(syntax_from_cursor(
                 arena, src, nl_pos, cursor, config, highlights,
             ));
         }
@@ -846,11 +907,19 @@ fn atom_from_cursor<'a>(
     nl_pos: &NewlinePositions,
     cursor: &mut ts::TreeCursor,
     highlights: &HighlightedNodeIds,
-) -> &'a Syntax<'a> {
+) -> Vec<&'a Syntax<'a>> {
     let node = cursor.node();
     let position = nl_pos.from_offsets(node.start_byte(), node.end_byte());
     let mut content = &src[node.start_byte()..node.end_byte()];
 
+    // The C and C++ grammars have a '\n' node with the
+    // preprocessor. This isn't useful for difftastic, because it's
+    // not visible, but leads us to highlight unchanged lines that
+    // happen to have preceding newline node.
+    if node.kind() == "\n" {
+        return vec![];
+    }
+
     // JSX trims whitespace at the beginning and end of text nodes.
     // TODO: match the exact trimming behaviour used in React.
     //
@@ -872,7 +941,7 @@ fn atom_from_cursor<'a>(
         AtomKind::Normal
     };
 
-    Syntax::new_atom(arena, position, content, highlight)
+    vec![Syntax::new_atom(arena, position, content, highlight)]
 }
 
 #[cfg(test)]
